"use strict";
/**
 * @author Riccardo La Marca
 *
 * @brief Generic commands:
 *  - help  [Shows the helper message for each command or a specified one]
 *  - clear [Clear the screen content]
 *  - quit  [Quit the application]
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.quit_command = exports.clear_command = exports.help_command = void 0;
const utils = __importStar(require("../utils"));
const HelpCommand = async (data, state) => {
    // First we need to filter the commands given the input ones
    const filter_result = state.commands.filter((value) => data.includes(value.command));
    if (data.length > 0) {
        const command = filter_result[0];
        const separator_str = '-'.repeat(command.name.length);
        console.log(utils.FormatString("{0}\n{1}\n{2}", command.name, separator_str, command.help));
    }
    else {
        const commands = {};
        state.commands.forEach((v) => {
            const group = v.group || 'Generic';
            if (!(group in commands))
                commands[group] = [];
            commands[group].push(v);
        });
        for (const group in commands) {
            console.log(`####################### ${group} Commands List #######################\n`);
            commands[group].forEach((cmd) => {
                const separator_str = '-'.repeat(cmd.name.length);
                console.log(utils.FormatString("{0}\n{1}\n{2}", cmd.name, separator_str, cmd.help));
            });
            console.log();
        }
    }
    // Do not modify the state, but it must returns it
    return state;
};
// Help Command
exports.help_command = {
    group: 'Generic',
    name: 'Help Command',
    command: 'help',
    syntax: /\b\w+\b/g,
    callback: HelpCommand,
    help: 'help [cmd1 cmd2 ...] - Shows the helper string for each specified command.\n' +
        'If no command is specified, then consider all existing commands.\n',
};
const ClearCommand = async (_, state) => {
    console.clear();
    return state;
};
// Create Command - "Download" a problem instance locally
exports.clear_command = {
    group: 'Generic',
    name: 'Clear Command',
    command: 'clear',
    syntax: /^clear$/,
    callback: ClearCommand,
    help: 'clear - Clear the screen content.\n'
};
exports.quit_command = {
    group: 'Generic',
    name: 'Quit Command',
    command: 'quit',
    syntax: /^quit$/,
    callback: async (_, state) => state,
    help: 'quit - Quit the application.\n'
};
